@@ -1,5 +1,5 @@
 // generator.hpp
-// Copyright (c) 2007-2008 Ben Hanson (http://www.benhanson.net/)
+// Copyright (c) 2007-2009 Ben Hanson (http://www.benhanson.net/)
 //
 // Distributed under the Boost Software License, Version 1.0. (See accompanying
 // file licence_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
@@ -26,30 +26,35 @@ template<typename CharT, typename Traits = char_traits<CharT> >
 class basic_generator
 {
 public:
-    typedef typename basic_state_machine<CharT>::size_t_vector size_t_vector;
+    typedef typename detail::internals::size_t_vector size_t_vector;
     typedef basic_rules<CharT> rules;
 
-    static void build (const rules &rules_, basic_state_machine<CharT> &state_machine_)
+    static void build (const rules &rules_,
+        basic_state_machine<CharT> &state_machine_)
     {
         std::size_t index_ = 0;
         std::size_t size_ = rules_.statemap ().size ();
         node_ptr_vector node_ptr_vector_;
+        detail::internals &internals_ = const_cast<detail::internals &>
+            (state_machine_.data ());
+        bool seen_BOL_assertion_ = false;
+        bool seen_EOL_assertion_ = false;
 
         state_machine_.clear ();
 
         for (; index_ < size_; ++index_)
         {
-            state_machine_._lookup->push_back (0);
-            state_machine_._lookup->back () = new size_t_vector;
-            state_machine_._dfa_alphabet.push_back (0);
-            state_machine_._dfa->push_back (0);
-            state_machine_._dfa->back () = new size_t_vector;
+            internals_._lookup->push_back (0);
+            internals_._lookup->back () = new size_t_vector;
+            internals_._dfa_alphabet.push_back (0);
+            internals_._dfa->push_back (0);
+            internals_._dfa->back () = new size_t_vector;
         }
 
-        for (index_ = 0, size_ = state_machine_._lookup->size ();
+        for (index_ = 0, size_ = internals_._lookup->size ();
             index_ < size_; ++index_)
         {
-            state_machine_._lookup[index_]->resize (sizeof (CharT) == 1 ?
+            internals_._lookup[index_]->resize (sizeof (CharT) == 1 ?
                 num_chars : num_wchar_ts, dead_state_index);
 
             if (!rules_.regexes ()[index_].empty ())
@@ -58,26 +63,41 @@ public:
                 index_set_vector set_mapping_;
                 // syntax tree
                 detail::node *root_ = build_tree (rules_, index_,
-                    node_ptr_vector_, state_machine_._lookup[index_],
-                    set_mapping_, state_machine_._dfa_alphabet[index_],
-                    state_machine_._seen_BOL_assertion,
-                    state_machine_._seen_EOL_assertion);
+                    node_ptr_vector_, internals_, set_mapping_);
 
                 build_dfa (root_, set_mapping_,
-                    state_machine_._dfa_alphabet[index_],
-                    *state_machine_._dfa[index_]);
+                    internals_._dfa_alphabet[index_],
+                    *internals_._dfa[index_]);
+
+                if (internals_._seen_BOL_assertion)
+                {
+                    seen_BOL_assertion_ = true;
+                }
+
+                if (internals_._seen_EOL_assertion)
+                {
+                    seen_EOL_assertion_ = true;
+                }
+
+                internals_._seen_BOL_assertion = false;
+                internals_._seen_EOL_assertion = false;
             }
         }
+
+        internals_._seen_BOL_assertion = seen_BOL_assertion_;
+        internals_._seen_EOL_assertion = seen_EOL_assertion_;
     }
 
     static void minimise (basic_state_machine<CharT> &state_machine_)
     {
-        const std::size_t machines_ = state_machine_._dfa->size ();
+        detail::internals &internals_ = const_cast<detail::internals &>
+            (state_machine_.data ());
+        const std::size_t machines_ = internals_._dfa->size ();
 
         for (std::size_t i_ = 0; i_ < machines_; ++i_)
         {
-            const std::size_t dfa_alphabet_ = state_machine_._dfa_alphabet[i_];
-            size_t_vector *dfa_ = state_machine_._dfa[i_];
+            const std::size_t dfa_alphabet_ = internals_._dfa_alphabet[i_];
+            size_t_vector *dfa_ = internals_._dfa[i_];
 
             if (dfa_alphabet_ != 0)
             {
@@ -118,13 +138,14 @@ protected:
 
     static detail::node *build_tree (const rules &rules_,
         const std::size_t state_, node_ptr_vector &node_ptr_vector_,
-        size_t_vector *lookup_, index_set_vector &set_mapping_,
-        std::size_t &dfa_alphabet_, bool &seen_BOL_assertion_,
-        bool &seen_EOL_assertion_)
+        detail::internals &internals_, index_set_vector &set_mapping_)
     {
+        size_t_vector *lookup_ = internals_._lookup[state_];
         const typename rules::string_deque_deque &regexes_ =
             rules_.regexes ();
         const typename rules::id_vector_deque &ids_ = rules_.ids ();
+        const typename rules::id_vector_deque &unique_ids_ =
+            rules_.unique_ids ();
         const typename rules::id_vector_deque &states_ = rules_.states ();
         typename rules::string_deque::const_iterator regex_iter_ =
             regexes_[state_].begin ();
@@ -132,6 +153,8 @@ protected:
             regexes_[state_].end ();
         typename rules::id_vector::const_iterator ids_iter_ =
             ids_[state_].begin ();
+        typename rules::id_vector::const_iterator unique_ids_iter_ =
+            unique_ids_[state_].begin ();
         typename rules::id_vector::const_iterator states_iter_ =
             states_[state_].begin ();
         const typename rules::string &regex_ = *regex_iter_;
@@ -144,15 +167,17 @@ protected:
 
         build_macros (token_map_, macrodeque_, macromap_,
             rules_.flags (), rules_.locale (), node_ptr_vector_,
-            seen_BOL_assertion_, seen_EOL_assertion_);
+            internals_._seen_BOL_assertion, internals_._seen_EOL_assertion);
 
         detail::node *root_ = parser::parse (regex_.c_str (),
-            regex_.c_str () + regex_.size (), *ids_iter_, *states_iter_,
-            rules_.flags (), rules_.locale (), node_ptr_vector_, macromap_,
-            token_map_, seen_BOL_assertion_, seen_EOL_assertion_);
+            regex_.c_str () + regex_.size (), *ids_iter_, *unique_ids_iter_,
+            *states_iter_, rules_.flags (), rules_.locale (), node_ptr_vector_,
+            macromap_, token_map_, internals_._seen_BOL_assertion,
+            internals_._seen_EOL_assertion);
 
         ++regex_iter_;
         ++ids_iter_;
+        ++unique_ids_iter_;
         ++states_iter_;
         tree_vector_.push_back (root_);
 
@@ -164,16 +189,18 @@ protected:
 
             root_ = parser::parse (regex_.c_str (),
                 regex_.c_str () + regex_.size (), *ids_iter_,
-                *states_iter_, rules_.flags (), rules_.locale (),
-                node_ptr_vector_, macromap_, token_map_,
-                seen_BOL_assertion_, seen_EOL_assertion_);
+                *unique_ids_iter_, *states_iter_, rules_.flags (),
+                rules_.locale (), node_ptr_vector_, macromap_, token_map_,
+                internals_._seen_BOL_assertion,
+                internals_._seen_EOL_assertion);
             tree_vector_.push_back (root_);
             ++regex_iter_;
             ++ids_iter_;
+            ++unique_ids_iter_;
             ++states_iter_;
         }
 
-        if (seen_BOL_assertion_)
+        if (internals_._seen_BOL_assertion)
         {
             // Fixup BOLs
             typename detail::node::node_vector::iterator iter_ =
@@ -237,7 +264,7 @@ protected:
             }
         }
 
-        dfa_alphabet_ = token_list_->size () + dfa_offset;
+        internals_._dfa_alphabet[state_] = token_list_->size () + dfa_offset;
         return root_;
     }
 
@@ -254,11 +281,11 @@ protected:
             const typename rules::string &name_ = iter_->first;
             const typename rules::string &regex_ = iter_->second;
             detail::node *node_ = parser::parse (regex_.c_str (),
-                regex_.c_str () + regex_.size (), 0, 0, flags_,
+                regex_.c_str () + regex_.size (), 0, 0, 0, flags_,
                 locale_, node_ptr_vector_, macromap_, token_map_,
                 seen_BOL_assertion_, seen_EOL_assertion_);
             macro_iter_pair map_iter_ = macromap_.
-                insert (macro_pair (name_, 0));
+                insert (macro_pair (name_, (detail::node const*)0));
 
             map_iter_.first->second = node_;
         }
@@ -292,8 +319,9 @@ protected:
                 iter_ != end_; ++iter_)
             {
                 equivset *equivset_ = *iter_;
-                const std::size_t transition_ = closure (&equivset_->_followpos,
-                    seen_sets_, seen_vectors_, hash_vector_, dfa_alphabet_, dfa_);
+                const std::size_t transition_ = closure
+                    (&equivset_->_followpos, seen_sets_, seen_vectors_,
+                    hash_vector_, dfa_alphabet_, dfa_);
 
                 if (transition_ != npos)
                 {
@@ -335,10 +363,12 @@ protected:
 
     static std::size_t closure (typename detail::node::node_vector *followpos_,
         node_set_vector &seen_sets_, node_vector_vector &seen_vectors_,
-        size_t_vector &hash_vector_, const std::size_t size_, size_t_vector &dfa_)
+        size_t_vector &hash_vector_, const std::size_t size_,
+        size_t_vector &dfa_)
     {
         bool end_state_ = false;
         std::size_t id_ = 0;
+        std::size_t unique_id_ = npos;
         std::size_t state_ = 0;
         std::size_t hash_ = 0;
 
@@ -352,8 +382,8 @@ protected:
             followpos_->begin (), end_ = followpos_->end ();
             iter_ != end_; ++iter_)
         {
-            closure_ex (*iter_, end_state_, id_, state_, set_ptr_.get (),
-                vector_ptr_.get (), hash_);
+            closure_ex (*iter_, end_state_, id_, unique_id_, state_,
+                set_ptr_.get (), vector_ptr_.get (), hash_);
         }
 
         bool found_ = false;
@@ -390,6 +420,7 @@ protected:
             {
                 dfa_[old_size_] |= end_state;
                 dfa_[old_size_ + id_index] = id_;
+                dfa_[old_size_ + unique_id_index] = unique_id_;
                 dfa_[old_size_ + state_index] = state_;
             }
         }
@@ -398,8 +429,8 @@ protected:
     }
 
     static void closure_ex (detail::node *node_, bool &end_state_,
-        std::size_t &id_, std::size_t &state_, node_set *set_ptr_,
-        node_vector *vector_ptr_, std::size_t &hash_)
+        std::size_t &id_, std::size_t &unique_id_, std::size_t &state_,
+        node_set *set_ptr_, node_vector *vector_ptr_, std::size_t &hash_)
     {
         const bool temp_end_state_ = node_->end_state ();
 
@@ -409,6 +440,7 @@ protected:
             {
                 end_state_ = true;
                 id_ = node_->id ();
+                unique_id_ = node_->unique_id ();
                 state_ = node_->lexer_state ();
             }
         }
@@ -479,7 +511,7 @@ protected:
                     }
                     else
                     {
-                        iter_ = lhs_->insert (++iter_, 0);
+                        iter_ = lhs_->insert (++iter_, (charset*)0);
                         *iter_ = overlap_.release ();
 
                         // VC++ 6 Hack:
@@ -621,7 +653,7 @@ protected:
                     }
                     else
                     {
-                        iter_ = lhs_->insert (++iter_, 0);
+                        iter_ = lhs_->insert (++iter_, (equivset*)0);
                         *iter_ = overlap_.release ();
 
                         // VC++ 6 Hack:
@@ -702,12 +734,14 @@ protected:
         if (!found_)
         {
             node_ptr_vector_->push_back (0);
-            node_ptr_vector_->back () = new detail::leaf_node (bol_token, true);
+            node_ptr_vector_->back () = new detail::leaf_node
+                (bol_token, true);
 
             detail::node *lhs_ = node_ptr_vector_->back ();
 
             node_ptr_vector_->push_back (0);
-            node_ptr_vector_->back () = new detail::leaf_node (null_token, true);
+            node_ptr_vector_->back () = new detail::leaf_node
+                (null_token, true);
 
             detail::node *rhs_ = node_ptr_vector_->back ();
 
@@ -793,6 +827,7 @@ protected:
 
                 new_ptr_[end_state_index] = ptr_[end_state_index];
                 new_ptr_[id_index] = ptr_[id_index];
+                new_ptr_[unique_id_index] = ptr_[unique_id_index];
                 new_ptr_[state_index] = ptr_[state_index];
                 new_ptr_[bol_index] = lookup_ptr_[ptr_[bol_index]];
                 new_ptr_[eol_index] = lookup_ptr_[ptr_[eol_index]];
