@@ -17,7 +17,7 @@
 //      [
 //          exported key - class name*
 //          tracking level - always/never
-//          file version
+//          class version
 //      ]
 //
 //      if tracking
@@ -42,7 +42,7 @@ namespace archive {
 //This should never ever change. note that is not an std::string
 // string.
 BOOST_ARCHIVE_DECL(const char *) 
-ARCHIVE_SIGNATURE(){
+BOOST_ARCHIVE_SIGNATURE(){
     return "serialization::archive";
 }
 
@@ -52,10 +52,16 @@ ARCHIVE_SIGNATURE(){
 // 1 - initial version
 // 2 - made address tracking optional
 // 3 - numerous changes - can't guarentee compatibility with previous versions
- 
+// 4 - Boost 1.34
+//     added register_version to properly support versioning for collections 
+// 5 - Boost 1.36
+//     changed serialization of collections: adding version even for primitive
+//     types caused backwards compatibility breaking change in 1.35
+// 6 - Boost 1.41, serializing collection sizes as std::size_t
+
 BOOST_ARCHIVE_DECL(unsigned char)
-ARCHIVE_VERSION(){
-    return 3;
+BOOST_ARCHIVE_VERSION(){
+    return 6;
 }
 
 } // namespace archive
